# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/tpu/topology.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tensorflow/core/protobuf/tpu/topology.proto\x12\x0etensorflow.tpu\"\x99\x01\n\x12TPUHardwareFeature\x12N\n\x11\x65mbedding_feature\x18\x01 \x01(\x0e\x32\x33.tensorflow.tpu.TPUHardwareFeature.EmbeddingFeature\"3\n\x10\x45mbeddingFeature\x12\x0f\n\x0bUNSUPPORTED\x10\x00\x12\x06\n\x02V1\x10\x01\x12\x06\n\x02V2\x10\x02\"\xb6\x01\n\rTopologyProto\x12\x12\n\nmesh_shape\x18\x01 \x03(\x05\x12\x11\n\tnum_tasks\x18\x02 \x01(\x05\x12 \n\x18num_tpu_devices_per_task\x18\x03 \x01(\x05\x12\x1a\n\x12\x64\x65vice_coordinates\x18\x04 \x03(\x05\x12@\n\x14tpu_hardware_feature\x18\x05 \x01(\x0b\x32\".tensorflow.tpu.TPUHardwareFeatureB\x03\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.tpu.topology_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _TPUHARDWAREFEATURE._serialized_start=64
  _TPUHARDWAREFEATURE._serialized_end=217
  _TPUHARDWAREFEATURE_EMBEDDINGFEATURE._serialized_start=166
  _TPUHARDWAREFEATURE_EMBEDDINGFEATURE._serialized_end=217
  _TOPOLOGYPROTO._serialized_start=220
  _TOPOLOGYPROTO._serialized_end=402
# @@protoc_insertion_point(module_scope)
