# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/control_flow.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tensorflow/core/protobuf/control_flow.proto\x12\ntensorflow\"\x96\x01\n\tValuesDef\x12\x0e\n\x06values\x18\x01 \x03(\t\x12\x42\n\x0f\x65xternal_values\x18\x02 \x03(\x0b\x32).tensorflow.ValuesDef.ExternalValuesEntry\x1a\x35\n\x13\x45xternalValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x83\x01\n\x15\x43ontrolFlowContextDef\x12/\n\tcond_ctxt\x18\x01 \x01(\x0b\x32\x1a.tensorflow.CondContextDefH\x00\x12\x31\n\nwhile_ctxt\x18\x02 \x01(\x0b\x32\x1b.tensorflow.WhileContextDefH\x00\x42\x06\n\x04\x63txt\"\xc4\x01\n\x0e\x43ondContextDef\x12\x14\n\x0c\x63ontext_name\x18\x01 \x01(\t\x12\x11\n\tpred_name\x18\x02 \x01(\t\x12\x12\n\npivot_name\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x05\x12)\n\nvalues_def\x18\x05 \x01(\x0b\x32\x15.tensorflow.ValuesDef\x12:\n\x0fnested_contexts\x18\x06 \x03(\x0b\x32!.tensorflow.ControlFlowContextDef\"\xf5\x02\n\x0fWhileContextDef\x12\x14\n\x0c\x63ontext_name\x18\x01 \x01(\t\x12\x1b\n\x13parallel_iterations\x18\x02 \x01(\x05\x12\x11\n\tback_prop\x18\x03 \x01(\x08\x12\x13\n\x0bswap_memory\x18\x04 \x01(\x08\x12\x12\n\npivot_name\x18\x05 \x01(\t\x12\x1b\n\x13pivot_for_pred_name\x18\x06 \x01(\t\x12\x1b\n\x13pivot_for_body_name\x18\x07 \x01(\t\x12\x17\n\x0floop_exit_names\x18\x08 \x03(\t\x12\x18\n\x10loop_enter_names\x18\n \x03(\t\x12)\n\nvalues_def\x18\t \x01(\x0b\x32\x15.tensorflow.ValuesDef\x12\x1f\n\x17maximum_iterations_name\x18\x0b \x01(\t\x12:\n\x0fnested_contexts\x18\x0c \x03(\x0b\x32!.tensorflow.ControlFlowContextDefB\x89\x01\n\x18org.tensorflow.frameworkB\x11\x43ontrolFlowProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.control_flow_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\021ControlFlowProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _VALUESDEF_EXTERNALVALUESENTRY._options = None
  _VALUESDEF_EXTERNALVALUESENTRY._serialized_options = b'8\001'
  _VALUESDEF._serialized_start=60
  _VALUESDEF._serialized_end=210
  _VALUESDEF_EXTERNALVALUESENTRY._serialized_start=157
  _VALUESDEF_EXTERNALVALUESENTRY._serialized_end=210
  _CONTROLFLOWCONTEXTDEF._serialized_start=213
  _CONTROLFLOWCONTEXTDEF._serialized_end=344
  _CONDCONTEXTDEF._serialized_start=347
  _CONDCONTEXTDEF._serialized_end=543
  _WHILECONTEXTDEF._serialized_start=546
  _WHILECONTEXTDEF._serialized_end=919
# @@protoc_insertion_point(module_scope)
